/*
 nRF24 RC Car RX
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 2/11/2013
 Last Modified: 10/11/2013
 
 Receives the commands from the TX side, controls a servo and motor. Has a watchdog reset timeout of 4 seconds
 The servo output also goes to a dixon voltage doubler for the 2 high side N mosfets which is controlled by a 
 optoisolator which is switched on or off by PA1 or PA2. The low side N mosfets are also switch on by PA1 or PA2.
 
 */
 
// ATtiny24/44/84 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//         PB0  2|    |13  PA0 Servo for left/right
//     LED PB1  3|    |12  PA1 Motor forward control
//   Reset PB3  4|    |11  PA2 Motor reverse control
//      CE PB2  5|    |10  PA3 
//     CSN PA7  6|    |9   PA4 SCK
//      MI PA6  7|    |8   PA5 MO
//               +----+

#define F_CPU 8000000 // 8 MHz

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <string.h>
#include "setup.c"
#include "mirf.c"

uint8_t data_in[2]; // First byte: Left/Right, Second byte: Forward/Reverse
uint8_t nrfTimeout = 0; // A timeout for stopping the motor
uint8_t prepareMotorForStart = true; // Used if we need to restart timer0
uint8_t motorDirection = MOTOR_FORWARD;

int main(void) {
	setup();
	mirf_init();
	_delay_ms(50);
	mirf_config();
	
	// Power up the nRF24L01
	RX_POWERUP;
	mirf_flush_rx_tx(); // Flush TX/RX
	mirf_CE_hi; // Start listening
	
	while(1) {
		if (mirf_status() & (1<<RX_DR)) { // Check if a packet arrived
			// Reset the watchdog
			wdt_reset();
			
			// Clear the data_in
			data_in[LEFT_RIGHT_CTRL] = 0;
			data_in[FORWARD_REVERSE_CTRL] = 0;
			
			mirf_CSN_lo; // Pull down chip select
			spi_transfer(R_RX_PAYLOAD); // Send cmd to read rx payload
			spi_read_data(data_in, mirf_PAYLOAD); // Read payload
			mirf_CSN_hi; // Pull up chip select
			mirf_config_register(STATUS,(1<<RX_DR)); // Reset status register
			
			// Left/Right control
			int leftrightReading = data_in[LEFT_RIGHT_CTRL] * 2;
			if (leftrightReading > 300) { // Right
				leftrightReading += 18; // Fine tuning
			}
			else if (leftrightReading < 200) { // Left
				leftrightReading -= 30; // Fine tuning
			}
			OCR1A = 750 - leftrightReading;
			
			// Motor control
			int forwardreverseReading = data_in[FORWARD_REVERSE_CTRL];
			if (forwardreverseReading > 150) { // Forward
				// Fine tune the motor control
				uint8_t calcOcr = 0;
				for (int x = 0; x < (forwardreverseReading-150); x++) {
					calcOcr += 2;
				}
				if (calcOcr > 170) { 
					calcOcr = 250;
				}
				
				OCR0A = calcOcr;
				motorDirection = MOTOR_FORWARD;
				
				if (prepareMotorForStart == 1) { // Turn on timer once
					TCNT0 = 0;
					TCCR0A = (1<<WGM00); // PWM, Phase Correct
					TCCR0B = (1<<CS02);
					sbi(TIMSK0, OCIE0A); // Output Compare A Match Interrupt Enable
					prepareMotorForStart = 0;
				}
			}
			else if (forwardreverseReading < 100) { // Reverse
				// Fine tune the motor control
				uint8_t calcOcr = 0;
				for (int x = 0; x < (100 - forwardreverseReading); x++) {
					calcOcr += 2;
				}
				if (calcOcr > 170) { 
					calcOcr = 250;
				}
				
				OCR0A = calcOcr;
				motorDirection = MOTOR_REVERSE;
				
				if (prepareMotorForStart == true) { // Turn on timer once
					TCNT0 = 0;
					TCCR0A = (1<<WGM00); // PWM, Phase Correct
					TCCR0B = (1<<CS02);
					sbi(TIMSK0, OCIE0A); // Output Compare A Match Interrupt Enable
					prepareMotorForStart = false;
				}
			}
			else { // Disable the timer/motor if no forward/reverse command is being issued
				prepareMotorForStart = true;
				TCNT0 = 0;
				TCCR0A = 0; // PWM, Phase Correct
				TCCR0B = 0;
				OCR0A = 0; // Sets duty cycle to 0%
				cbi(TIMSK0, OCIE0A); // Output Compare A Match Interrupt Enable
				PORTA &= ~(1<<PA1);
				PORTA &= ~(1<<PA2);
			}
			
			mirf_flush_rx_tx(); // Flush TX/RX
			mirf_CE_hi; // Start listening
			nrfTimeout = 0;
		}
		else {
			if (nrfTimeout == 255) { // Disable the timer/motor if we haven't heard from the transmitter in a while
				prepareMotorForStart = true;
				TCNT0 = 0;
				TCCR0A = 0; // PWM, Phase Correct
				TCCR0B = 0;
				OCR0A = 0; // Sets duty cycle to 0%
				cbi(TIMSK0, OCIE0A); // Output Compare A Match Interrupt Enable
				PORTA &= ~(1<<PA1);
				PORTA &= ~(1<<PA2);
			}
			nrfTimeout++;
		}
		
		_delay_us(50);
	}
}

// Servo control - left/right
ISR(TIM1_COMPA_vect) { 
	PORTA ^= (1<<PA0);
}

// Motor control - forward/reverse
ISR(TIM0_COMPA_vect) {
	if (motorDirection == MOTOR_FORWARD) {
		PORTA &= ~(1<<PA2);
		PORTA ^= (1<<PA1);
	}
	else {
		PORTA &= ~(1<<PA1);
		PORTA ^= (1<<PA2);
	}
}
